﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using HeuristicLab.Problems.DataAnalysis;
using System.Globalization;

namespace Tests {
  [TestClass]
  public class ShapeConstraintsParserTest {
    [TestMethod]
    public void TestFunctionRangeConstraints() {
      System.Threading.Thread.CurrentThread.CurrentCulture = CultureInfo.InvariantCulture;
      {
        var constraint = ShapeConstraintsParser.ParseFunctionRangeConstraint("f in [1 .. 2]");
        Assert.AreEqual(string.Empty, constraint.Variable);
        Assert.AreEqual(0, constraint.NumberOfDerivations);
        Assert.AreEqual(false, constraint.IsDerivative);
        Assert.AreEqual(1.0, constraint.Weight);
        Assert.AreEqual(0, constraint.Regions.Count);
        Assert.AreEqual(1.0, constraint.Interval.LowerBound);
        Assert.AreEqual(2.0, constraint.Interval.UpperBound);
      }
      {
        var constraint = ShapeConstraintsParser.ParseFunctionRangeConstraint("f in [1 2]");
        Assert.AreEqual(string.Empty, constraint.Variable);
        Assert.AreEqual(0, constraint.NumberOfDerivations);
        Assert.AreEqual(false, constraint.IsDerivative);
        Assert.AreEqual(1.0, constraint.Weight);
        Assert.AreEqual(0, constraint.Regions.Count);
        Assert.AreEqual(1.0, constraint.Interval.LowerBound);
        Assert.AreEqual(2.0, constraint.Interval.UpperBound);
      }
      {
        var constraint = ShapeConstraintsParser.ParseFunctionRangeConstraint("f in [1; 2]");
        Assert.AreEqual(string.Empty, constraint.Variable);
        Assert.AreEqual(0, constraint.NumberOfDerivations);
        Assert.AreEqual(false, constraint.IsDerivative);
        Assert.AreEqual(1.0, constraint.Weight);
        Assert.AreEqual(0, constraint.Regions.Count);
        Assert.AreEqual(1.0, constraint.Interval.LowerBound);
        Assert.AreEqual(2.0, constraint.Interval.UpperBound);
      }
      {
        var constraint = ShapeConstraintsParser.ParseFunctionRangeConstraint("f in [1 .. 2] weight: 2.0");
        Assert.AreEqual(string.Empty, constraint.Variable);
        Assert.AreEqual(0, constraint.NumberOfDerivations);
        Assert.AreEqual(false, constraint.IsDerivative);
        Assert.AreEqual(2.0, constraint.Weight);
        Assert.AreEqual(0, constraint.Regions.Count);
        Assert.AreEqual(1.0, constraint.Interval.LowerBound);
        Assert.AreEqual(2.0, constraint.Interval.UpperBound);
      }
      {
        var constraint = ShapeConstraintsParser.ParseFunctionRangeConstraint("f in [1 .. 2], x in [2 .. 3], y in [4 .. 5]");
        Assert.AreEqual(string.Empty, constraint.Variable);
        Assert.AreEqual(0, constraint.NumberOfDerivations);
        Assert.AreEqual(false, constraint.IsDerivative);
        Assert.AreEqual(1.0, constraint.Weight);
        Assert.AreEqual(2, constraint.Regions.Count);
        Assert.AreEqual(2, constraint.Regions.GetInterval("x").LowerBound);
        Assert.AreEqual(3, constraint.Regions.GetInterval("x").UpperBound);
        Assert.AreEqual(4, constraint.Regions.GetInterval("y").LowerBound);
        Assert.AreEqual(5, constraint.Regions.GetInterval("y").UpperBound);
        Assert.AreEqual(1.0, constraint.Interval.LowerBound);
        Assert.AreEqual(2.0, constraint.Interval.UpperBound);
      }
      {
        var constraint = ShapeConstraintsParser.ParseFunctionRangeConstraint("f in [1 .. 2], x in [2 .. 3], y in [4 .. 5] weight: 2.0");
        Assert.AreEqual(string.Empty, constraint.Variable);
        Assert.AreEqual(0, constraint.NumberOfDerivations);
        Assert.AreEqual(false, constraint.IsDerivative);
        Assert.AreEqual(2.0, constraint.Weight);
        Assert.AreEqual(2, constraint.Regions.Count);
        Assert.AreEqual(2, constraint.Regions.GetInterval("x").LowerBound);
        Assert.AreEqual(3, constraint.Regions.GetInterval("x").UpperBound);
        Assert.AreEqual(4, constraint.Regions.GetInterval("y").LowerBound);
        Assert.AreEqual(5, constraint.Regions.GetInterval("y").UpperBound);
        Assert.AreEqual(1.0, constraint.Interval.LowerBound);
        Assert.AreEqual(2.0, constraint.Interval.UpperBound);
      }
    }
    [TestMethod]
    public void TestDerivationConstraints() {
      System.Threading.Thread.CurrentThread.CurrentCulture = CultureInfo.InvariantCulture;
      var variables = new string[] { "x", "var x" };
      {
        var constraint = ShapeConstraintsParser.ParseDerivationConstraint("df/dx in [1 .. 2]", variables);
        Assert.AreEqual("x", constraint.Variable);
        Assert.AreEqual(1, constraint.NumberOfDerivations);
        Assert.AreEqual(true, constraint.IsDerivative);
        Assert.AreEqual(1.0, constraint.Weight);
        Assert.AreEqual(0, constraint.Regions.Count);
        Assert.AreEqual(1.0, constraint.Interval.LowerBound);
        Assert.AreEqual(2.0, constraint.Interval.UpperBound);
      }
      {
        var constraint = ShapeConstraintsParser.ParseDerivationConstraint("d²f/dx² in [1 .. 2]", variables);
        Assert.AreEqual("x", constraint.Variable);
        Assert.AreEqual(2, constraint.NumberOfDerivations);
        Assert.AreEqual(true, constraint.IsDerivative);
        Assert.AreEqual(1.0, constraint.Weight);
        Assert.AreEqual(0, constraint.Regions.Count);
        Assert.AreEqual(1.0, constraint.Interval.LowerBound);
        Assert.AreEqual(2.0, constraint.Interval.UpperBound);
      }
      {
        var constraint = ShapeConstraintsParser.ParseDerivationConstraint("d² f / d x² in [1 .. 2]", variables);
        Assert.AreEqual("x", constraint.Variable);
        Assert.AreEqual(2, constraint.NumberOfDerivations);
        Assert.AreEqual(true, constraint.IsDerivative);
        Assert.AreEqual(1.0, constraint.Weight);
        Assert.AreEqual(0, constraint.Regions.Count);
        Assert.AreEqual(1.0, constraint.Interval.LowerBound);
        Assert.AreEqual(2.0, constraint.Interval.UpperBound);
      }
      {
        var constraint = ShapeConstraintsParser.ParseDerivationConstraint("d³ f / d 'var x'³ in [1 .. 2]", variables);
        Assert.AreEqual("var x", constraint.Variable);
        Assert.AreEqual(3, constraint.NumberOfDerivations);
        Assert.AreEqual(true, constraint.IsDerivative);
        Assert.AreEqual(1.0, constraint.Weight);
        Assert.AreEqual(0, constraint.Regions.Count);
        Assert.AreEqual(1.0, constraint.Interval.LowerBound);
        Assert.AreEqual(2.0, constraint.Interval.UpperBound);
      }
    }
  }
}
